from AbstractService import AbstractService
import Registry
from Version import Version
from Service import ServiceState
from Service import ServiceKind
from datatypes.LenzeDriveMeasurement import LenzeDriveMeasurement
from datatypes.AggregatedPlcEnergyMeasurement import AggregatedPlcEnergyMeasurement
from datatypes.DriveAiResult import DriveAiResult
from decimal import Decimal

class DrivePathPythonbasedAIByLenzeInterface(AbstractService):
    """Partially implemented service interface for "Drive path Python-based AI by Lenze"
       Generated by: EASy-Producer."""
    
    ingestor = None
    param_aiId = "Lenze-AI"
    
    def __init__(self):
        """Initializes the service.""" 
        super().__init__("DriveLenzePythonAi", "Drive path Python-based AI by Lenze", Version("0.1.0"), "Drive path Python-based AI by Lenze", True, ServiceKind.TRANSFORMATION_SERVICE)
        Registry.services['DriveLenzePythonAi'] = self
        self.registerMethods()
        
    def attachIngestor(self, ingestor):
        """Called by ServiceEnvironment to attach an individual type-generic ingestor, i.e., a function 
           that allows to asynchronously ingest result data back into stream processing. The ingestor
           cares for the type and the target stream.
           
           Parameters:
             - ingestor -- the ingestor function
           """ 
        self.ingestor = ingestor

    def ingest(self, data):
        """Ingests result data asynchronously into stream processing. Target stream and data type are 
           handled by the ingestor.
           
           Parameters:
             - data -- the data to ingest; if the type is unknown/no type translator registered, nothing will happen
           """ 
        if self.ingestor is not None:
            self.ingestor(data)

    def setState(self, state:ServiceState):
        """Changes the state. [R133c]
        
        Parameters:
          - newState -- the new state (ServiceState)
        """
        super().setState(state)
        if (state==ServiceState.STARTING):
            self.start()
        if (state==ServiceState.STOPPING):
            self.stop()

    def start(self):
        """Called when the server shall start.
        """
        pass

    def stop(self):
        """Called when the server shall stop.
        """
        pass

    def notifyReconfigured(self, values: dict):
        """Called to inform the implementing class that a parameter reconfiguration happened.
           
           Parameters:
             - values -- the new param names/values
           """ 
        pass
    
    def reconfigure(self, values:dict):
        """Called by ServiceEnvironment to initialize service parameters or to reconfigure them at runtime.
           
           Parameters:
             - values -- the new param names/values
           """ 
        if "aiId" in values:
            self.param_aiId = values["aiId"]
        self.notifyReconfigured(values)
        
    def getAiId(self):
        """Returns the value of parameter aiId. May change at runtime upon reconfiguration.
           
           Result:
             - the value of the parameter
           """ 
    
        return self.param_aiId
    
    
    def processLenzeDriveMeasurement(self, data: LenzeDriveMeasurement):
        """Asynchronous data processing method. Use self.ingest(data) to pass the result back to the data stream.
           Possible output types: DriveAiResultImpl
    
        Parameters:
          - data -- the data to process
        """
        raise NotImplementedError
    def processAggregatedPlcEnergyMeasurement(self, data: AggregatedPlcEnergyMeasurement):
        """Asynchronous data processing method. Use self.ingest(data) to pass the result back to the data stream.
           Possible output types: DriveAiResultImpl
    
        Parameters:
          - data -- the data to process
        """
        raise NotImplementedError            
    def registerMethods(self):
        Registry.asyncTransformers['DriveLenzePythonAi_LenzeDriveMeasurement'] = self.processLenzeDriveMeasurement
        Registry.asyncTransformers['DriveLenzePythonAi_AggregatedPlcEnergyMeasurement'] = self.processAggregatedPlcEnergyMeasurement
